@@ -1,6 +1,7 @@
+#-*- perl -*-
 
 package MIME::EncWords;
-use 5.005;
+require 5.005;
 
 =head1 NAME
 
@@ -120,7 +121,7 @@ if (MIME::Charset::USE_ENCODE) {
 #------------------------------
 
 ### The package version, both in 1.23 style *and* usable by MakeMaker:
-$VERSION = '1.010';
+$VERSION = '1.011';
 
 ### Public Configuration Attributes
 $Config = {
@@ -336,7 +337,7 @@ sub decode_mimewords {
 	    }
 
 	    if (scalar(@tokens) and
-		lc($charset) eq lc($tokens[-1]->[1]) and
+		lc($charset || "") eq lc($tokens[-1]->[1] || "") and
 		resolve_alias($charset) and
 		(!${tokens[-1]}[2] and !$language or
 		 lc(${tokens[-1]}[2]) eq lc($language))) { # Concat words if possible.
@@ -385,7 +386,7 @@ sub decode_mimewords {
     if ($Params{Detect7bit} ne "NO") {
 	foreach my $t (@tokens) {
 	    unless ($t->[0] =~ $UNSAFE or $t->[1]) {
-		my $charset = &MIME::Charset::_detect_7bit_charset($t->[0]);
+		my $charset = MIME::Charset::_detect_7bit_charset($t->[0]);
 		if ($charset and $charset ne &MIME::Charset::default()) {
 		    $t->[1] = $charset;
 		}
@@ -416,11 +417,9 @@ sub _convert($$$$) {
     croak "unsupported charset ``".$cset->as_string."''"
 	unless $cset->decoder or $cset->as_string eq "_UNICODE_";
 
-    my $preserveerr = $@;
-
+    local($@);
     $charset = MIME::Charset->new($charset, Mapping => $mapping);
     if ($charset->as_string and $charset->as_string eq $cset->as_string) {
-	$@ = $preserveerr;
 	return $s;
     }
     # build charset object to transform string from $charset to $cset.
@@ -453,8 +452,6 @@ sub _convert($$$$) {
     } elsif ($charset->decoder) {
 	$converted = $charset->encode($s);
     }
-
-    $@ = $preserveerr;
     return $converted;
 }
 
@@ -536,7 +533,7 @@ B<**>
 RAW may be a Unicode string when Unicode/multibyte support is enabled
 (see L<MIME::Charset/USE_ENCODE>).
 Furthermore, RAW may be a reference to that returned
-by L<"decode_mimewords"> on array context.  In latter case "Charset"
+by L</decode_mimewords> on array context.  In latter case "Charset"
 option (see below) will be overridden (see also a note below).
 
 B<Note>:
@@ -593,7 +590,7 @@ B<**>
 
 A Sequence to fold encoded lines.  The default is C<"\n">.
 If empty string C<""> is specified, encoded-words exceeding line length
-(see L<"MaxLineLen"> below) will be splitted by SPACE.
+(see L</MaxLineLen> below) will be splitted by SPACE.
 
 B<Note>:
 B<*>
@@ -695,13 +692,15 @@ sub encode_mimewords  {
 
 		# workaround for ``ASCII transformation'' charsets
 		my $u = $w;
-		if ($charsetobj->as_string =~ /$ASCIITRANS/ and $u =~ /[+~]/) {
-		    if ($charsetobj->decoder) {
-			$u = $charsetobj->decode($u);
-		    } elsif (!MIME::Charset::USE_ENCODE) { # for pre-Encode env.
-			$u = "X$u";
-		    } else { # NOTREACHED
-			croak __PACKAGE__.": Bug in encode_mimewords";
+		if ($charsetobj->as_string =~ /$ASCIITRANS/) {
+		    if (MIME::Charset::USE_ENCODE) {
+			if (is_utf8($w) or $w =~ /$WIDECHAR/) {
+			    $w = $charsetobj->undecode($u);
+			} else {
+			    $u = $charsetobj->decode($w);
+			}
+		    } elsif ($w =~ /[+~]/) { #FIXME: for pre-Encode environment
+		        $u = "x$w";
 		    }
 		}
 		if (scalar(@words)) {
@@ -984,7 +983,7 @@ sub _clip_unsafe {
     while ($shorter < $longer) {
 	my $cur = int(($shorter + $longer + 1) / 2);
 	my $enc = substr($ustr, 0, $cur);
-	if (MIME::Charset::USE_ENCODE) {
+	if (MIME::Charset::USE_ENCODE ne '') {
 	    $enc = $charset->undecode($enc);
 	}
 	my $elen = $charset->encoded_header_len($enc, $encoding);
@@ -1003,7 +1002,8 @@ sub _clip_unsafe {
 	eval {
 	    ($fenc, $renc) =
 		(substr($ustr, 0, $shorter), substr($ustr, $shorter));
-	    if (MIME::Charset::USE_ENCODE) {
+	    if (MIME::Charset::USE_ENCODE ne '') {
+		# FIXME: croak if $renc =~ /^\p{M}/
 		$fenc = $charset->undecode($fenc, FB_CROAK());
 		$renc = $charset->undecode($renc, FB_CROAK());
 	    }
@@ -1074,9 +1074,9 @@ sub _getparams {
 =head2 Configuration Files
 B<**>
 
-Built-in defaults of option parameters for L<"decode_mimewords">
+Built-in defaults of option parameters for L</decode_mimewords>
 (except 'Charset' option) and
-L<"encode_mimewords"> can be overridden by configuration files:
+L</encode_mimewords> can be overridden by configuration files:
 F<MIME/Charset/Defaults.pm> and F<MIME/EncWords/Defaults.pm>.
 For more details read F<MIME/EncWords/Defaults.pm.sample>.
 
@@ -1102,7 +1102,7 @@ L<MIME::Words> module that was written by:
 Other stuff are rewritten or added by:
     Hatuka*nezumi - IKEDA Soji <hatuka(at)nezumi.nu>.
 
-All rights reserved.  This program is free software; you can redistribute
+This program is free software; you can redistribute
 it and/or modify it under the same terms as Perl itself.
 
 =cut
